function [E, p, q] = depthMapEps(R, lambda, eps, p, q)

[h, w] = size(R); % height and width of image
E = zeros(h,w);
    
% Initialise p and q
pTmp = p;
qTmp = q;

done = false;
iter = 1;

while ~done
    % Compute partial derivatives
    [Rp, Rq] = gradient(R);
    
    % Compute averages of p and q using 4 nearest neighbours
    pAvg = zeros(size(p));
    qAvg = zeros(size(q));
    
    for x=2:h-1
        for y=2:w-1            
            pAvg(x,y) = (p(x+1,y) + p(x-1,y) + p(x,y+1) + p(x,y-1))/4;
            qAvg(x,y) = (q(x+1,y) + q(x-1,y) + q(x,y+1) + q(x,y-1))/4;
            
            % Update p and q
            pTmp(x,y) = pAvg(x,y) + (E(x,y)-R(x,y))*Rp(x,y)/(4*lambda);
            qTmp(x,y) = qAvg(x,y) + (E(x,y)-R(x,y))*Rq(x,y)/(4*lambda);
        end
    end
    
    cp = fft2(pTmp);
    cq = fft2(qTmp);
    
    cpq = zeros(h,w);
    
    for x=1:h
        for y=1:w
            cpq(x,y) = (-i*x*cp(x,y)-i*y*cq(x,y))/(x^2+y^2); % 9.27
        end
    end
    
    cpq(~isfinite(cpq)) = 0;
    E = abs(ifft2(cpq)); % 9.26
    [pTmp, qTmp] = gradient(E);
    
	% Compute the difference between the previous and the updated p and q
    diffP = abs(p-pTmp);
    diffQ = abs(q-qTmp);
    
	% Find the maximum in each matrice
    maxDiffP = max(diffP(:));
    maxDiffQ = max(diffQ(:));
    
	% Compare to stopping criteria
    if maxDiffP < eps && maxDiffQ < eps
        done = true;
    else
        iter = iter + 1;
    end
    
    p = pTmp;
    q = qTmp;
    
end

display(iter);

colormap hot;
imagesc(E);