function [E, p, q] = depthMapIter(R, lambda, iter, p, q)

[h, w] = size(R); % height and width of image
E = zeros(h,w);

for epoch=1:iter
    % Compute partial derivatives
    [Rp, Rq] = gradient(R);
    
    % Compute averages of p and q using 4 nearest neighbours
    pAvg = zeros(size(p));
    qAvg = zeros(size(q));
    
    for x=2:h-1
        for y=2:w-1            
            pAvg(x,y) = (p(x+1,y) + p(x-1,y) + p(x,y+1) + p(x,y-1))/4;
            qAvg(x,y) = (q(x+1,y) + q(x-1,y) + q(x,y+1) + q(x,y-1))/4;
            
            % Update p and q
            p(x,y) = pAvg(x,y) + (E(x,y)-R(x,y))*Rp(x,y)/(4*lambda);
            q(x,y) = qAvg(x,y) + (E(x,y)-R(x,y))*Rq(x,y)/(4*lambda);
        end
    end
    
    cp = fft2(p);
    cq = fft2(q);
    
    cpq = zeros(h,w);
    
    for x=1:h
        for y=1:w
            cpq(x,y) = (-i*x*cp(x,y)-i*y*cq(x,y))/(x^2+y^2); % 9.27
        end
    end
    
    cpq(~isfinite(cpq)) = 0;
    E = abs(ifft2(cpq)); % 9.26
    [p, q] = gradient(E);
end

colormap hot;
imagesc(E);